==> facility-2.16.18-linux-amd64.tar.zst: release-v2.16.18/packages/facility-server/app/database/integrity.js <==
import config from 'config';
import { log } from '@tamanu/shared/services/logging';
import { isSyncTriggerDisabled } from '@tamanu/shared/dataMigrations';
import { CentralServerConnection } from '../sync';

export async function performDatabaseIntegrityChecks(context) {
  if (await isSyncTriggerDisabled(context.sequelize)) {
    throw Error("Sync Trigger is disabled in the database.");
  }

  // run in a transaction so any errors roll back all changes
  await context.sequelize.transaction(async () => {
    await ensureHostMatches(context);
    await ensureFacilityMatches(context);
  });
}

/*
 * ensureHostMatches
 */
async function ensureHostMatches(context) {
  const { LocalSystemFact } = context.models;
  const centralServer = new CentralServerConnection(context);
  const configuredHost = centralServer.host;
  const lastHost = await LocalSystemFact.get('syncHost');

  if (!lastHost) {
    await LocalSystemFact.set('syncHost', centralServer.host);
    return;
  }

  if (lastHost !== configuredHost) {
    throw new Error(
      `integrity check failed: sync.host mismatch: read ${configuredHost} from config, but already connected to ${lastHost} (you may need to drop and recreate the database, change the config back, or if you're 100% sure, remove the "syncHost" key from the "local_system_fact" table)`,
    );
  }
}

/*
 * ensureFacilityMatches
 */
async function ensureFacilityMatches(context) {
  const { LocalSystemFact } = context.models;
  const configuredFacility = config.serverFacilityId;
  const lastFacility = await LocalSystemFact.get('facilityId');

  if (!lastFacility) {
    await performInitialIntegritySetup(context);
    return;
  }

  if (lastFacility !== configuredFacility) {
    // if the facility doesn't match, error
    throw new Error(
      `integrity check failed: serverFacilityId mismatch: read ${configuredFacility} from config, but already registered as ${lastFacility} (you may need to drop and recreate the database, change the config back, or if you're 100% sure, remove the "facilityId" key from the "local_system_fact" table)`,
    );
  }
}

async function performInitialIntegritySetup(context) {
  const centralServer = new CentralServerConnection(context);
  log.info(`Verifying sync connection to ${centralServer.host}...`);

  const { token, facility } = await centralServer.connect();

  if (!token) {
    throw new Error('Could not obtain valid token from central server.');
  }

  if (!facility) {
    throw new Error(
      `Configured serverFacilityId ${config.serverFacilityId} not recognised by central server`,
    );
  }

  // We've ensured that our immutable config stuff is valid -- save it!
  const { LocalSystemFact } = context.models;
  await LocalSystemFact.set('facilityId', facility.id);

  log.info(`Verified with central server as ${facility.name}`);
}

==> facility-2.19.7-linux-amd64.tar.zst: release-v2.19.7/packages/facility-server/app/database/integrity.js <==
import config from 'config';
import { log } from '@tamanu/shared/services/logging';
import { isSyncTriggerDisabled } from '@tamanu/shared/dataMigrations';
import { selectFacilityIds } from '@tamanu/shared/utils/configSelectors';
import { CentralServerConnection } from '../sync';

export async function performDatabaseIntegrityChecks(context) {
  if (await isSyncTriggerDisabled(context.sequelize)) {
    throw Error('Sync Trigger is disabled in the database.');
  }

  // run in a transaction so any errors roll back all changes
  await context.sequelize.transaction(async () => {
    await ensureHostMatches(context);
    await ensureFacilityMatches(context);
  });
}

/*
 * ensureHostMatches
 */
async function ensureHostMatches(context) {
  const { LocalSystemFact } = context.models;
  const centralServer = new CentralServerConnection(context);
  const configuredHost = centralServer.host;
  const lastHost = await LocalSystemFact.get('syncHost');

  if (!lastHost) {
    await LocalSystemFact.set('syncHost', centralServer.host);
    return;
  }

  if (lastHost !== configuredHost) {
    throw new Error(
      `integrity check failed: sync.host mismatch: read ${configuredHost} from config, but already connected to ${lastHost} (you may need to drop and recreate the database, change the config back, or if you're 100% sure, remove the "syncHost" key from the "local_system_fact" table)`,
    );
  }
}

/*
 * ensureFacilityMatches
 */
async function ensureFacilityMatches(context) {
  const { LocalSystemFact } = context.models;
  const configuredFacilities = selectFacilityIds(config);
  const lastFacilities = await LocalSystemFact.get('facilityIds');

  if (!lastFacilities) {
    await performInitialIntegritySetup(context);
    return;
  }

  // ensure both arrays contain the same set of facility ids
  const match = JSON.parse(lastFacilities).every(facilityId =>
    configuredFacilities.includes(facilityId),
  );
  if (!match) {
    // if the facility doesn't match, error
    throw new Error(
      `integrity check failed: serverFacilityId mismatch: read ${configuredFacilities} from config, but already registered as ${lastFacilities} (you may need to drop and recreate the database, change the config back, or if you're 100% sure, remove the "facilityIds" key from the "local_system_fact" table)`,
    );
  }
}

async function performInitialIntegritySetup(context) {
  const centralServer = new CentralServerConnection(context);
  log.info(`Verifying sync connection to ${centralServer.host}...`);

  const { token, serverFacilityIds } = await centralServer.connect();

  if (!token) {
    throw new Error('Could not obtain valid token from central server.');
  }

  // We've ensured that our immutable config stuff is valid -- save it!
  const { LocalSystemFact } = context.models;
  const facilityIdsString = JSON.stringify(serverFacilityIds);
  await LocalSystemFact.set('facilityIds', facilityIdsString);

  log.info(`Verified with central server as facilities ${facilityIdsString}`);
}
